<?php
/**
 * @brief		Member Filter Extension
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Indisposable
 * @since		13 Mar 2019
 */

namespace IPS\indisposable\extensions\core\MemberFilter;

/* To prevent PHP errors (extending class does not exist) revealing path */

use IPS\Helpers\Form\Radio;
use IPS\Helpers\Form\YesNo;

if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
    header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
    exit;
}

/**
 * Member Filter Extension
 */
class _MemberFilter
{
    /**
     * Determine if the filter is available in a given area
     *
     * @param	string	$area	Area to check
     * @return	bool
     */
    public function availableIn( $area )
    {
        return \in_array( $area, array( 'bulkmail', 'group_promotions' ) );
    }

    /**
     * Get Setting Field
     *
     * @param	mixed	$criteria	Value returned from the save() method
     * @return	array 	Array of form elements
     */
    public function getSettingField( $criteria )
    {
        $options = array( 'any' => 'any', 'active' => 'indisposable_mf_active', 'inactive' => 'indisposable_mf_inactive' );

        return [
            new Radio( 'indisposable_mf_filter', $criteria['indisposable_mf_filter'] ?? 'any', FALSE, [ 'options' => $options ] )
        ];
    }

    /**
     * Save the filter data
     *
     * @param	array	$post	Form values
     * @return	mixed			False, or an array of data to use later when filtering the members
     * @throws \LogicException
     */
    public function save( $post )
    {
        return !empty( $post['indisposable_mf_filter'] ) ? [ 'indisposable_mf_filter' => $post['indisposable_mf_filter'] ] : 'any';
    }

    /**
     * Get where clause to add to the member retrieval database query
     *
     * @param	mixed				$data	The array returned from the save() method
     * @return	array|NULL			Where clause - must be a single array( "clause", ...binds )
     */
    public function getQueryWhereClause( $data )
    {
        if ( !empty( $data['indisposable_mf_filter'] ) and ( $data['indisposable_mf_filter'] === 'inactive' ) )
        {
            return [ 'indisposable_members.member_id IS NULL' ];
        }

        // For consistency, but why, why in the world would someone want to ONLY e-mail members with disposable e-mails?
        if ( !empty( $data['indisposable_mf_filter'] ) and ( $data['indisposable_mf_filter'] === 'active' ) )
        {
            return [ 'indisposable_members.member_id IS NOT NULL' ];
        }
    }

    /**
     * Callback for member retrieval database query
     * Can be used to set joins
     *
     * @param	mixed			$data   The array returned from the save() method
     * @param	\IPS\Db\Select	$query  The query
     * @return	void
     */
    public function queryCallback( $data, &$query )
    {
        $query->join( 'indisposable_members', 'core_members.member_id=indisposable_members.member_id' );
    }

    /**
     * Determine if a member matches specified filters
     *
     * @note	This is only necessary if availableIn() includes group_promotions
     * @param	\IPS\Member	$member		Member object to check
     * @param	array 		$filters	Previously defined filters
     * @param	object|NULL	$object		Calling class
     * @return	bool
     */
    public function matches( \IPS\Member $member, $filters, $object=NULL )
    {
        if ( empty( $filters['indisposable_mf_filter'] ) or ( $filters['indisposable_mf_filter'] === 'any' ) )
        {
            return TRUE;
        }

        if ( !empty( $filters['indisposable_mf_filter'] ) and ( $filters['indisposable_mf_filter'] === 'active' ) )
        {
            return $member->isDisposable();
        }

        if ( !empty( $filters['indisposable_mf_filter'] ) and ( $filters['indisposable_mf_filter'] === 'inactive' ) )
        {
            return !$member->isDisposable();
        }

        return FALSE;
    }
}