<?php
/**
 * @brief		Member Sync
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Indisposable
 * @since		04 Mar 2019
 */

namespace IPS\indisposable\extensions\core\MemberSync;

/* To prevent PHP errors (extending class does not exist) revealing path */

use IPS\Request;

if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Member Sync
 */
class _MemberSync
{
	/**
	 * Member account has been created
	 *
	 * @param	$member	\IPS\Member	New member account
	 * @return	void
	 */
	public function onCreateAccount( $member )
	{
	    // This is for when "Block" mode is disabled and we're just logging registrations
	    $member->isDisposable( 'registration' );
	}
	
	/**
	 * Member has validated
	 *
	 * @param	\IPS\Member	$member		Member validated
	 * @return	void
	 */
	public function onValidate( $member )
	{
	
	}
	
	/**
	 * Member has logged on
	 *
	 * @param	\IPS\Member	$member		Member that logged in
	 * @return	void
	 */
	public function onLogin( $member )
	{
	    // Are they using a disposable e-mail, and are we asking them nicely to change?
	    if ( $member->isDisposable() and ( \IPS\Settings::i()->indisposable_member_action === 'prompt' ) )
        {
            // Since we're redirecting to a splash page instead of where the user originally logged in,
            // we want to store their original destination for the "continue to site" link
            if ( method_exists( \IPS\Request::i(), 'referrer' ) and \IPS\Request::i()->referrer( FALSE, TRUE ) )
            {
                $ref = \IPS\Request::i()->referrer( FALSE, TRUE );
            }
            elseif( !empty( $_SERVER['HTTP_REFERER'] ) )
            {
                $_ref = \IPS\Http\Url::createFromString( $_SERVER['HTTP_REFERER'] );
                $ref = ( $_ref instanceof \IPS\Http\Url\Internal and ( !isset( $_ref->queryString['do'] ) or $_ref->queryString['do'] != 'validating' ) ) ? $_ref : \IPS\Http\Url::internal('');
            }
            else
            {
                $ref = \IPS\Http\Url::internal( '' );
            }

            // Make sure we don't stupidly redirect the user back to the login screen
            if ( !( $ref instanceof \IPS\Http\Url\Internal ) or ( isset( $ref->hiddenQueryString['controller'] ) and ( $ref->hiddenQueryString['controller'] === 'login' ) ) )
            {
                $ref = \IPS\Http\Url::internal( '' );
            }

            \IPS\Request::i()->setCookie( 'indisposableRef', (string)$ref );
            \IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=core&module=system&controller=register&do=disposableEmailSplash', 'front', 'register' )->setQueryString( 'ref', (string)$ref ) );
        }
	}
	
	/**
	 * Member has logged out
	 *
	 * @param	\IPS\Member		$member			Member that logged out
	 * @param	\IPS\Http\Url	$returnUrl	    The URL to send the user back to
	 * @return	void
	 */
	public function onLogout( $member, $returnUrl )
	{
	
	}
	
	/**
	 * Member account has been updated
	 *
	 * @param	$member		\IPS\Member	Member updating profile
	 * @param	$changes	array		The changes
	 * @return	void
	 */
	public function onProfileUpdate( $member, $changes )
	{
	
	}
	
	/**
	 * Member is flagged as spammer
	 *
	 * @param	$member	\IPS\Member	The member
	 * @return	void
	 */
	public function onSetAsSpammer( $member )
	{
		
	}
	
	/**
	 * Member is unflagged as spammer
	 *
	 * @param	$member	\IPS\Member	The member
	 * @return	void
	 */
	public function onUnSetAsSpammer( $member )
	{
		
	}
	
	/**
	 * Member is merged with another member
	 *
	 * @param	\IPS\Member	$member		Member being kept
	 * @param	\IPS\Member	$member2	Member being removed
	 * @return	void
	 */
	public function onMerge( $member, $member2 )
	{
		$member->isDisposable( 'merge' );
	}
	
	/**
	 * Member is deleted
	 *
	 * @param	$member	\IPS\Member	The member
	 * @return	void
	 */
	public function onDelete( $member )
	{
        \IPS\Db::i()->delete( 'indisposable_members', [ 'member_id=?', $member->member_id ] );
        \IPS\Db::i()->delete( 'indisposable_logs', [ 'member_id=?', $member->member_id ] );
	}

	/**
	 * Email address is changed
	 *
	 * @param	\IPS\Member	$member	The member
	 * @param 	string		$new	New email address
	 * @param 	string		$old	Old email address
	 * @return	void
	 */
	public function onEmailChange( $member, $new, $old )
	{
	    if ( \IPS\indisposable\Validator::i()->isDisposable( $new ) )
        {
            \IPS\indisposable\Logger::log( $new, 'email_change', $member );

            if ( ( \IPS\Settings::i()->indisposable_action === 'block' ) )
            {
                throw new \DomainException( 'form_email_disposable' );
            }

            $member->isDisposable();
        }
	    else
        {
            \IPS\Db::i()->delete( 'indisposable_members', [ 'member_id=?', $member->member_id ] );
        }
	}

	/**
	 * Password is changed
	 *
	 * @param	\IPS\Member	$member	The member
	 * @param 	string		$new	New password
	 * @return	void
	 */
	public function onPassChange( $member, $new )
	{

	}
}