<?php
/**
 * @brief		Background Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Indisposable
 * @since		14 Mar 2019
 */

namespace IPS\indisposable\extensions\core\Queue;

/* To prevent PHP errors (extending class does not exist) revealing path */

use IPS\Task\Queue\OutOfRangeException;

if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Background Task
 */
class _Queue
{
    /**
     * @brief Number of content items to rebuild per cycle
     */
    public $rebuild	= \IPS\REBUILD_SLOW;

	/**
	 * Parse data before queuing
	 *
	 * @param	array	$data
	 * @return	array
	 */
	public function preQueueData( $data )
	{
        \IPS\indisposable\Validator::i()->purgeDisposableMembers();
        $data['memberCount'] = (int) \IPS\Db::i()->select( 'count(*)', 'core_members', "email<>''" )->first();
        $data['processed'] = 0;

        if( $data['memberCount'] == 0 )
        {
            return null;
        }

		return $data;
	}

	/**
	 * Run Background Task
	 *
	 * @param	mixed						$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int							$offset	Offset
	 * @return	int							New offset
	 * @throws	\IPS\Task\Queue\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function run( &$data, $offset )
	{
        if ( !$data['memberCount'] )
        {
            throw new OutOfRangeException();
        }

	    // Our join is so we can skip members we already know are disposable and speed up the rebuild process
        $select = \IPS\Db::i()->select(
            'member_id, email', 'core_members', "email<>''", 'member_id',
            [ $offset, $this->rebuild ]
        );

        $last = NULL;
        foreach ( $select as $row )
        {
            // Make sure the e-mail field isn't empty (incomplete social registration, SSO, etc.)
            if ( empty( $row['email'] ) )
            {
                continue;
            }

            try
            {
                if ( \IPS\indisposable\Validator::i()->isDisposable( $row['email'] ) )
                {
                    \IPS\Db::i()->insert( 'indisposable_members', [ 'member_id' => $row['member_id'] ], FALSE, TRUE );
                }

                $data['processed']++;
                $last = $row['member_id'];
            }
            catch ( \Exception $e ) {}
        }

        if ( $last === NULL )
        {
            throw new OutOfRangeException();
        }

        return ( $offset + $this->rebuild );
	}

	/**
	 * Get Progress
	 *
	 * @param	mixed					$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int						$offset	Offset
	 * @return	array( 'text' => 'Doing something...', 'complete' => 50 )	Text explaining task and percentage complete
	 * @throws	\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function getProgress( $data, $offset )
	{
		return array(
		    'text' => \IPS\Member::loggedIn()->language()->addToStack( 'indisposable_queue_progress' ),
            'complete' => $data['memberCount'] ? ( round( 100 * ( $data['processed'] / $data['memberCount'] ), 2 ) ) : 100
        );
	}

	/**
	 * Perform post-completion processing
	 *
	 * @param	array	$data
	 * @return	void
	 */
	public function postComplete( $data )
	{

	}
}