<?php /** @noinspection PhpDocMissingThrowsInspection */

namespace IPS\indisposable;

class _Validator extends \IPS\Patterns\Singleton
{
    /**
     * @var string  API endpoint
     */
    protected static $apiEndpoint = 'https://open.kickbox.com/v1/disposable/';

    /**
     * @var array   Cached queries
     */
    protected $cache = [];

    /**
     * @brief	Database Table
     */
    public static $databaseTable = 'indisposable_cache';

    /**
     * @brief	Database Table
     */
    public static $membersDatabaseTable = 'indisposable_members';

    /**
     * A cached list of well-known valid e-mail services
     * @var string[]
     * @link https://github.com/mailcheck/mailcheck/wiki/List-of-Popular-Domains
     */
    public static $whitelist = [
        /* Common domains */
        "aol.com", "att.net", "comcast.net", "facebook.com", "gmail.com", "gmx.com", "googlemail.com",
        "google.com", "hotmail.com", "hotmail.co.uk", "mac.com", "me.com", "mail.com", "msn.com",
        "live.com", "sbcglobal.net", "verizon.net", "yahoo.com", "yahoo.co.uk",

        /* Other global domains */
        "email.com", "fastmail.fm", "games.com" /* AOL */, "gmx.net", "hush.com", "hushmail.com", "icloud.com",
        "iname.com", "inbox.com", "lavabit.com", "love.com" /* AOL */, "outlook.com", "pobox.com", "protonmail.com",
        "rocketmail.com" /* Yahoo */, "safe-mail.net", "wow.com" /* AOL */, "ygm.com" /* AOL */,
        "ymail.com" /* Yahoo */, "zoho.com", "yandex.com",

        /* United States ISP domains */
        "bellsouth.net", "charter.net", "cox.net", "earthlink.net", "juno.com",

        /* British ISP domains */
        "btinternet.com", "virginmedia.com", "blueyonder.co.uk", "freeserve.co.uk", "live.co.uk",
        "ntlworld.com", "o2.co.uk", "orange.net", "sky.com", "talktalk.co.uk", "tiscali.co.uk",
        "virgin.net", "wanadoo.co.uk", "bt.com",

        /* Domains used in Asia */
        "sina.com", "sina.cn", "qq.com", "naver.com", "hanmail.net", "daum.net", "nate.com", "yahoo.co.jp", "yahoo.co.kr", "yahoo.co.id", "yahoo.co.in", "yahoo.com.sg", "yahoo.com.ph", "163.com", "126.com", "aliyun.com", "foxmail.com",

        /* French ISP domains */
        "hotmail.fr", "live.fr", "laposte.net", "yahoo.fr", "wanadoo.fr", "orange.fr", "gmx.fr", "sfr.fr", "neuf.fr", "free.fr",

        /* German ISP domains */
        "gmx.de", "hotmail.de", "live.de", "online.de", "t-online.de" /* T-Mobile */, "web.de", "yahoo.de",

        /* Italian ISP domains */
        "libero.it", "virgilio.it", "hotmail.it", "aol.it", "tiscali.it", "alice.it", "live.it", "yahoo.it", "email.it", "tin.it", "poste.it", "teletu.it",

        /* Russian ISP domains */
        "mail.ru", "rambler.ru", "yandex.ru", "ya.ru", "list.ru",

        /* Belgian ISP domains */
        "hotmail.be", "live.be", "skynet.be", "voo.be", "tvcablenet.be", "telenet.be",

        /* Argentinian ISP domains */
        "hotmail.com.ar", "live.com.ar", "yahoo.com.ar", "fibertel.com.ar", "speedy.com.ar", "arnet.com.ar",

        /* Domains used in Mexico */
        "yahoo.com.mx", "live.com.mx", "hotmail.es", "hotmail.com.mx", "prodigy.net.mx",

        /* Domains used in Brazil */
        "yahoo.com.br", "hotmail.com.br", "outlook.com.br", "uol.com.br", "bol.com.br", "terra.com.br", "ig.com.br", "itelefonica.com.br", "r7.com", "zipmail.com.br", "globo.com", "globomail.com", "oi.com.br"
    ];

    /**
     * @brief	Singleton Instances
     */
    protected static $instance = NULL;

    /**
     * Check and see if this is a disposable e-mail address
     *
     * @param   string $email
     * @param   bool   $incrementUseCounter
     * @return  array|bool|mixed
     */
    public function isDisposable( $email, $incrementUseCounter=FALSE )
    {
        if ( empty( $email ) )
        {
            throw new \InvalidArgumentException( 'No valid e-mail provided' );
        }

        $split = $this->_splitEmail( $email );

        // Check our hardcoded cache first
        if ( \in_array( $split['host'], static::$whitelist ) )
        {
            return FALSE;
        }

        // Check if we've already cached this query
        $cache = $this->checkCache( $split['host'] );
        if ( !\is_null( $cache ) )
        {
            if ( $incrementUseCounter and $cache['disposable'] )
            {
                $this->incrementUseCount( $cache );
            }

            return $cache['disposable'];
        }

        // No local cache? Let's validate it remotely
        $request = \IPS\Http\Url::external( static::$apiEndpoint . urlencode( $split['host'] ) )->request();
        try
        {
            $get        = $request->get();
            $response   = $get->decodeJson();

            if ( !isset( $response['disposable'] ) )
            {
                $errorData = [
                    'email'                 => $email,
                    'requestUrl'            => (string)\IPS\Http\Url::external( static::$apiEndpoint . urlencode( $split['host'] ) ),
                    'httpResponseVersion'   => $get->httpResponseVersion,
                    'httpResponseCode'      => $get->httpResponseCode,
                    'httpResponseText'      => $get->httpResponseText,
                    'httpHeaders'           => $get->httpHeaders,
                    'response'              => $response
                ];

                \IPS\Log::log( 'Malformed kickbox validation response: ' . json_encode( $errorData ) );
                return FALSE;
            }

        }
        catch ( \IPS\Http\Request\Exception $e )
        {
            \IPS\Log::log( $e );
            return FALSE; // If our query fails, just return false. Otherwise, we end up blocking all registrations.
        }

        $this->addCache( $split['host'], (bool)$response['disposable'] );
        return (bool)$response['disposable'];
    }

    /**
     * Is this a G-Mail alias of an existing member?
     * @param $email
     * @return \IPS\Member[]|bool
     */
    public function isAlias( $email )
    {
        if ( empty( $email ) )
        {
            throw new \InvalidArgumentException( 'No valid e-mail provided' );
        }

        $split = $this->_splitEmail( $email );
        $googleMail = FALSE;

        // Make sure this is a GMail address first
        if ( \in_array( $split['host'], [ 'gmail.com', 'googlemail.com' ] ) )
        {
            $googleMail = TRUE;
        }
        // It might be a custom GSuite domain if it's not already in the whitelist
        elseif ( !\in_array( $split['host'], static::$whitelist ) )
        {
            $mxResults = [];
            if ( \getmxrr( $split['host'], $mxResults ) )
            {
                if ( $mxResults and ( \strlen( $mxResults[0] ) >= 10 ) and ( \substr( $mxResults[0], -10, 10 ) === 'google.com' ) )
                {
                    $googleMail = TRUE;
                }
            }
        }

        // We only support alias checks for Google
        if ( !$googleMail )
        {
            return FALSE;
        }

        $filteredUser = \explode( '+', \str_replace( '.', '', $split['user'] ), 2 )[0];
        $filteredHost = \str_replace( '.', '', $split['host'] );

        /**
         * First search for an existing e-mail that may just be missing period discriminators
         * e.g., the following two e-mails are the same on GMail
         * - john.doe@gmail.com
         * - johndoe@gmail.com
         */
        $aliases = [];

        $select = \IPS\Db::i()->select(
            '*', 'core_members', [ "REPLACE( email, '.', '' ) = ?", \str_replace( '.', '', "{$filteredUser}@{$filteredHost}" ) ]
        );
        foreach ( $select as $member )
        {
            $aliases[ $member['email'] ] = \IPS\Member::constructFromData( $member );
        }


        /**
         * Next, search for + style aliases.
         * e.g, the following two e-mails are the same on GMail
         * - john.doe@gmail.com
         * - john.doe+foo@gmail.com
         */
        $select = \IPS\Db::i()->select(
            '*', 'core_members', [ "REPLACE( email, '.', '' ) LIKE ?", "{$filteredUser}+%@{$filteredHost}" ]
        );

        foreach ( $select as $member )
        {
            $aliases[ $member['email'] ] = \IPS\Member::constructFromData( $member );
        }

        // Don't include a literal e-mail match (i.e. existing accounts IPS will already pick-up)
        unset( $aliases[ $email ] );

        return $aliases;
    }

    /**
     * Check and see if we have
     *
     * @param   string  $host
     * @return  array|bool|mixed
     */
    protected function checkCache( $host )
    {
        // Have we already made this request?
        if ( !empty( $this->cache[ $host ] ) )
        {
            return $this->cache[ $host ];
        }

        // Have we recently confirmed the domain is okay?
        try
        {
            $cache = \IPS\Data\Cache::i()->getWithExpire( "indisposable_cache_{$host}", TRUE );
            return $this->cache[ $host ] = $cache;
        }
        catch ( \OutOfRangeException $e ) {}

        // Still here? Check our permanent list of disposable addresses
        try
        {
            $cache = \IPS\Db::i()->select( '*', static::$databaseTable, [ 'host=?', $host ] )->first();
        }
        catch (\UnderflowException $e)
        {
            return NULL;
        }

        return $this->cache[ $host ] = array_merge( $cache, [ 'disposable' => TRUE ] );
    }

    /**
     * Log validation status locally so we don't waste HTTP requests
     *
     * @param   string  $host
     * @param   bool    $disposable
     */
    protected function addCache( $host, $disposable = TRUE )
    {
        $this->cache[ $host ] = $disposable;

        // For false results we only want a temporary cache (new disposable addresses may not be added yet)
        if ( $disposable === FALSE )
        {
            \IPS\Data\Cache::i()->storeWithExpire( "indisposable_cache_{$host}", [ 'disposable' => FALSE ], \IPS\DateTime::create()->add( new \DateInterval( 'P1D' ) ), TRUE );
            return;
        }

        // Otherwise: once a disposable e-mail, always a disposable e-mail
        \IPS\Db::i()->insert(
            static::$databaseTable, [
            'host'      => $host,
            'uses'      => 1,
            'first_use' => time(),
            'last_use'  => time()
        ]);
    }

    /**
     * Increment a cached domains use count
     * @param   array   $cache
     */
    protected function incrementUseCount( array $cache )
    {
        \IPS\Db::i()->update( static::$databaseTable, [ 'uses' => $cache['uses'] + 1, 'last_use' => time() ], [ 'id=?', $cache['id'] ] );
    }

    /**
     * Purge the disposable members database table
     * This is done prior to rebuilding
     */
    public function purgeDisposableMembers()
    {
        \IPS\Db::i()->delete( static::$membersDatabaseTable );
    }

    /**
     * Split an e-mail into its domain and user components
     * @param   string  $email
     * @return  array
     */
    final protected function _splitEmail( $email )
    {
        $split  = explode( '@', $email );
        $host   = array_pop( $split );
        $user   = implode( '@', $split );

        return ['user' => $user, 'host' => $host];
    }
}